# config.py
import os
import torch

class Config:
    # 数据集路径
    RAW_DATA_PATH = os.path.join("data", "raw")
    PROCESSED_DATA_PATH = os.path.join("data", "processed", "processed_data.txt")
    
    # 模型路径
    PRETRAINED_MODEL_DIR = os.path.join("models", "pretrained","DeepSeek-R1-Distill-Qwen-1dot5B")  # 预训练模型目录
    TRAINED_MODEL_DIR = os.path.join("models", "trained","DeepSeek-R1-Distill-Qwen-1dot5B")        # 训练后模型保存目录
    # TRAINED_MODEL_DIR = os.path.join("models", "trained","DeepSeek-R1-Distill-Qwen-1dot5B-WindTurbine")        # 训练后模型保存目录

    # PRETRAINED_MODEL_DIR = os.path.join("models", "pretrained","DeepSeek-R1-Distill-Qwen-7B")  # 预训练模型目录
    # TRAINED_MODEL_DIR = os.path.join("models", "trained","DeepSeek-R1-Distill-Qwen-7B")        # 训练后模型保存目录

    # PRETRAINED_MODEL_DIR = os.path.join("models", "pretrained","DeepSeek-R1-Distill-Qwen-32B")  # 预训练模型目录
    # TRAINED_MODEL_DIR = os.path.join("models", "trained","DeepSeek-R1-Distill-Qwen-32B")        # 训练后模型保存目录

    # PRETRAINED_MODEL_DIR = os.path.join("models", "pretrained","DeepSeek-R1-Distill-Llama-8B")  # 预训练模型目录
    # TRAINED_MODEL_DIR = os.path.join("models", "trained","DeepSeek-R1-Distill-Llama-8B")        # 训练后模型保存目录
    RUN_MODEL_DIR=TRAINED_MODEL_DIR  # TRAINED_MODEL_DIR

    # 设备配置
    DEVICE_STR="cpu"  # 
    # DEVICE_STR="cuda" if torch.cuda.is_available() else "cpu"
    DEVICE=torch.device(DEVICE_STR)
    TORCH_DTYPE = torch.bfloat16 if DEVICE_STR == "cuda" else torch.float32
    
    # 训练参数
    USE_FP16 = True if DEVICE_STR=="cuda" else False # 是否使用混合精度训练，在CPU上无法有效使用FP16，且可能引发梯度问题，须关闭混合精度训练，即将USE_FP16设为False
    BATCH_SIZE = 16 # 32  # 每个训练批次中的样本数量。较大的批次大小可以提高训练速度，但会占用更多显存。
    EPOCHS = 10      # 训练的总轮数。每一轮会遍历整个数据集一次。
    LEARNING_RATE = 1e-4  # 1e-4 # 学习率，控制模型参数更新的步长。较小的学习率会使训练更稳定，但可能需要更多轮数才能收敛。
    SEQ_LENGTH = 9126  # 输入序列的最大长度。较长的序列可以捕捉更多上下文信息，但会增加计算和显存开销。
    NUM_WORKERS=4 
    
    # 运行参数
    TORCH_DTYPE=torch.float16
    GRADIENT_ACCUMULATION_STEPS = 4  # 梯度累积步数
    
    TOP_K = 50
    TOP_P = 0.95
    NUM_BEAMS = 1  # 禁用束搜索
    MAX_TOKENS = 512  # 50 # 生成文本时的最大长度（以 token 为单位）。控制生成文本的长度。
    MAX_INPUT_LENGTH = 2048  # 输入序列的最大长度
    MAX_NEW_TOKENS = 200  # 生成文本的最大长度（新增部分）范围200~512
    MAX_LENGTH = MAX_INPUT_LENGTH + MAX_NEW_TOKENS  # 总长度
    # 上下文管理
    MAX_CONTEXT_TOKENS = 2048    # 最大上下文token数
    MAX_HISTORY_TOKENS = 1024    # 历史记录最大token数

    TEMPERATURE = 0.7 # 0.7  # 控制生成文本的随机性。较低的温度（如 0.2）会使生成结果更确定，较高的温度（如 1.0）会使生成结果更多样化。
    DO_SAMPLE = True

    # 系统功能
    RUN_WARMUP = True           # 是否启用预热
    STREAMING = False  # 是否启用流式输出
    BATCH_ENABLED = True  # 是否启用批处理
    MODEL_LOAD_KWARGS = {}  # 额外加载参数