import os
import json
from src.config import Config

class DataProcessor:
    def __init__(self):
        self.raw_data_path = Config.RAW_DATA_PATH
        self.processed_data_path = Config.PROCESSED_DATA_PATH

    def generate_raw_data(self):
        """生成原始数据集"""
        raw_data = [
            # """格格巫是谁？他是《蓝精灵》中的主要反派角色——一名巫师，总是试图抓捕蓝精灵，用他们来炼制魔法药水。格格巫有一只忠实的猫伙伴，名叫阿兹猫吗？他们经常一起策划各种阴谋，但最终总是被蓝精灵们挫败。在一个风雨交加的夜晚，格格巫在他的实验室里忙碌着。他正在研究一种新的魔法药水，希望能够增强自己的力量。阿兹猫在一旁，好奇地注视着周围的环境。突然，实验室的门被风吹开，一道闪电划过天空，照亮了格格巫那张阴险的脸。格格巫的实验室里堆满了各种奇怪的物品：魔法书、药水瓶、干草茎和蜘蛛网。魔法书记载着古老的咒语和配方。药水瓶装着五颜六色的液体，有的还在冒泡。干草茎用于制作飞行药水的关键材料，蓝妹妹深入研究过的蓝妹妹之术似乎也有制魔法药力。格格巫一边翻看着魔法书，一边喃喃自语：“只要再抓到一个蓝精灵，我就能制成我的终极魔法药水了。”阿兹猫在一边打了个哈欠，似乎对主人的计划并不感兴趣。格格巫终于抓到了一个蓝精灵。""",
            """济公是谁？""",
            """济公，原名李修缘，是中国南宋时期的一位传奇僧人。他以不羁的生活方式和神奇的法力闻名，常常帮助穷苦百姓，惩治恶人。
Jigong, originally named Li Xiuyuan, was a legendary monk during the Southern Song Dynasty in China. He was known for his unconventional lifestyle and magical powers, often helping the poor and punishing the wicked.""",
            """济公常常穿着破烂的僧袍，手持破扇，行走于市井之间。他看似疯癫，实则智慧非凡。
Jigong often wore tattered monk robes and carried a broken fan, wandering among the common people. He appeared to be crazy, but in reality, he was extraordinarily wise.""",
            """有一次，济公遇到一个贪婪的富商，富商欺压百姓，无恶不作。济公用他的智慧教训了富商，让他悔过自新。
Once, Jigong encountered a greedy merchant who oppressed the people and committed all kinds of evil deeds. Jigong used his wisdom to teach the merchant a lesson, making him repent and reform.""",
            """济公的故事在中国民间广为流传，他被视为正义和智慧的象征。
The stories of Jigong are widely spread in Chinese folklore, and he is regarded as a symbol of justice and wisdom.""",
"""济公的名言是：“酒肉穿肠过，佛祖心中留。” 这句话表达了他不拘小节的生活态度和对佛法的深刻理解。
Jigong's famous saying is: "Wine and meat pass through the intestines, but Buddha remains in the heart." This phrase expresses his carefree attitude towards life and his deep understanding of Buddhism.""",
            """济公的传说不仅在中国流传，还被翻译成多种语言，传播到世界各地。
The legends of Jigong are not only popular in China but have also been translated into many languages and spread around the world."""
        ]
        with open(self.raw_data_path, "w", encoding="utf-8") as f:
            f.write("\n".join(raw_data))

    def process_data(self):
        """处理指定目录下的 .txt、.json、.jsonl 文件"""
        if not os.path.exists(self.raw_data_path):
            raise FileNotFoundError(f"Raw data directory not found at {self.raw_data_path}")
        
        processed_data:list[str]=[]

        # 遍历目录中的所有文件
        for filename in os.listdir(self.raw_data_path):
            file_path = os.path.join(self.raw_data_path, filename)

            # 处理 .txt 文件
            if filename.endswith('.txt'):
                """处理 .txt 文件"""
                with open(file_path, "r", encoding="utf-8") as f:
                    raw_data = f.read().splitlines()

                # 示例：简单的数据处理（分词、过滤等）
                file_data = [line.lower() for line in raw_data]
                processed_data.extend(file_data)

        self.save_data_to_file(processed_data,file_path)

            # # 处理 .json 文件
            # elif filename.endswith('.json'):
            #     self.process_json(file_path)

            # # 处理 .jsonl 文件
            # elif filename.endswith('.jsonl'):
            #     self.process_jsonl(file_path)

    def process_json(self, file_path):
        """处理 .json 文件"""
        with open(file_path, "r", encoding="utf-8") as f:
            raw_data = json.load(f)

        # 示例：简单的数据处理（转为小写等）
        processed_data = {key: value.lower() if isinstance(value, str) else value
                          for key, value in raw_data.items()}

        # 保存处理后的数据
        self.save_data_to_file(processed_data, file_path)

    def process_jsonl(self, file_path):
        """处理 .jsonl 文件"""
        processed_data = []
        with open(file_path, "r", encoding="utf-8") as f:
            for line in f:
                raw_data = json.loads(line.strip())
                # 示例：简单的数据处理（转为小写等）
                processed_data.append({key: value.lower() if isinstance(value, str) else value
                                       for key, value in raw_data.items()})

        # 保存处理后的数据
        self.save_data_to_file(processed_data, file_path)

    def save_data_to_file(self, data, file_path):
        """保存处理后的数据到文件"""
        os.makedirs(os.path.dirname(self.processed_data_path), exist_ok=True)

        # 如果是处理后的 .json 或 .jsonl，直接保存成 JSON 格式
        if file_path.endswith('.json') or file_path.endswith('.jsonl'):
            with open(self.processed_data_path, "w", encoding="utf-8") as f:
                if file_path.endswith('.json'):
                    json.dump(data, f, ensure_ascii=False, indent=4)
                elif file_path.endswith('.jsonl'):
                    for item in data:
                        f.write(json.dumps(item, ensure_ascii=False) + "\n")
        
        # 如果是处理后的 .txt，直接保存为文本文件
        elif file_path.endswith('.txt'):
            with open(self.processed_data_path, "w", encoding="utf-8") as f:
                f.write("\n".join(data))

        print(f"Processed data saved to {self.processed_data_path}")